package com.ruoyi.web.controller.business;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.InterestsNumberTable;
import com.ruoyi.system.service.IInterestsNumberTableService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 商户权益次数记录Controller
 * 
 * @author ruoyi
 * @date 2022-07-08
 */
@RestController
@RequestMapping("/system/numberTable")
public class InterestsNumberTableController extends BaseController
{
    @Autowired
    private IInterestsNumberTableService interestsNumberTableService;

    /**
     * 查询商户权益次数记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:table:list')")
    @GetMapping("/list")
    public TableDataInfo list(InterestsNumberTable interestsNumberTable)
    {
        startPage();
        List<InterestsNumberTable> list = interestsNumberTableService.selectInterestsNumberTableList(interestsNumberTable);
        return getDataTable(list);
    }

    /**
     * 导出商户权益次数记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:table:export')")
    @Log(title = "商户权益次数记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, InterestsNumberTable interestsNumberTable)
    {
        List<InterestsNumberTable> list = interestsNumberTableService.selectInterestsNumberTableList(interestsNumberTable);
        ExcelUtil<InterestsNumberTable> util = new ExcelUtil<InterestsNumberTable>(InterestsNumberTable.class);
        util.exportExcel(response, list, "商户权益次数记录数据");
    }

    /**
     * 获取商户权益次数记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:table:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(interestsNumberTableService.selectInterestsNumberTableById(id));
    }

    /**
     * 新增商户权益次数记录
     */
    @PreAuthorize("@ss.hasPermi('system:table:add')")
    @Log(title = "商户权益次数记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody InterestsNumberTable interestsNumberTable)
    {
        return toAjax(interestsNumberTableService.insertInterestsNumberTable(interestsNumberTable));
    }

    /**
     * 修改商户权益次数记录
     */
    @PreAuthorize("@ss.hasPermi('system:table:edit')")
    @Log(title = "商户权益次数记录", businessType = BusinessType.UPDATE)
    @PostMapping("/put")
    public AjaxResult edit(@RequestBody InterestsNumberTable interestsNumberTable)
    {
        return toAjax(interestsNumberTableService.updateInterestsNumberTable(interestsNumberTable));
    }

    /**
     * 删除商户权益次数记录
     */
    @PreAuthorize("@ss.hasPermi('system:table:remove')")
    @Log(title = "商户权益次数记录", businessType = BusinessType.DELETE)
	@GetMapping("/delete/{numberId}")
    public AjaxResult remove(@PathVariable Long numberId)
    {
        return toAjax(interestsNumberTableService.deleteInterestsNumberTableById(numberId));
    }
}
