package com.ruoyi.web.controller.system;

import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisService;
import com.ruoyi.common.exception.base.BaseException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.domain.SettledMerchants;
import com.ruoyi.system.service.ISettledMerchantsService;
import com.ruoyi.system.service.ISysLogininforService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysMenu;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginBody;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.framework.web.service.SysPermissionService;
import com.ruoyi.system.service.ISysMenuService;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;

import static com.ruoyi.common.constant.Constants.LOGIN_NUM;
import static com.ruoyi.framework.web.service.SysLoginService.checkStrongPwd;

/**
 * 登录验证
 * 
 * @author ruoyi
 */
@RestController
public class SysLoginController
{
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISettledMerchantsService settledMerchantsService;
    @Resource
    private RedisService redisService;

    /**
     * 登录方法
     * 
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody)
    {
        if ("1".equals(checkStrongPwd(loginBody.getPassword()))) {
            return AjaxResult.error("密码必须包含数字、大小写字母、特殊符号且大于8位");
        }
        SysUser user = userService.selectUserByUserName(loginBody.getUsername());
        String username = user.getUserName();

        if (!SecurityUtils.matchesPassword(loginBody.getPassword(), user.getPassword())) {
            if (!"admin".equals(username)){
                String num = redisService.getCacheObject(LOGIN_NUM + username);
                if(StringUtils.isEmpty(num)){
                    num = "0";
                }
                if(Integer.parseInt(num) == 5){
                    return AjaxResult.error("登录密码输入5次错误,现已锁定，3分钟后再次输入密码");
                }
                //存入redis登录错误次数
                redisService.setCacheObject(LOGIN_NUM + username, String.valueOf(Integer.parseInt(num)+1), Constants.USERNAME_EXPIRATION, TimeUnit.MINUTES);
            }
            return AjaxResult.error("用户不存在/密码错误");
        }

        //判断2个月内是否修改密码，排除admin
        if(StringUtils.isNotEmpty(username) && !"admin".equals(username)){
            String isUpdate = redisService.getCacheObject(Constants.PASSWORD_USER + username);
            if(StringUtils.isEmpty(isUpdate)){
                return AjaxResult.error("60天内为登录,需修改密码后重新登录");
            }
        }
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), loginBody.getPassword(), loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }


    @PostMapping("/loginApp")
    public AjaxResult loginApp(@RequestBody LoginBody loginBody)
    {
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.loginApp(loginBody.getUsername(), loginBody.getPassword(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        //根据userName查询用户
        SysUser user = userService.selectUserByUserName(loginBody.getUsername());
        if(user==null){
            return AjaxResult.error("账号不存在");
        }
        ajax.put("userId", user.getUserId());
        //获取商家的信息
        SettledMerchants settledMerchants = settledMerchantsService.selectByPhone(user.getUserName());
        if(!user.getUserName().equals("admin") && settledMerchants==null){
            return AjaxResult.error("该账号无绑定商户");
        }

        if(user.getUserName().equals("admin")){
            settledMerchants = new SettledMerchants();
            settledMerchants.setBusinessId(-1L);
            settledMerchants.setBusinessPhone(user.getPhonenumber());
        }
        ajax.put("settledMerchants", settledMerchants);
        return ajax;
    }

    /**
     * 获取用户信息
     * 
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo()
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }


    /**
     * 获取路由信息
     * 
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters()
    {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }

    /**
     * 产生uuid二维码
     */
    @PostMapping("/scanCode/getQrCode")
    public R<?> getQrCode() {
        String qrCode = tokenService.getQrCode();
        return R.ok(qrCode);
    }

    /**
     * 判断二维码是否被扫描
     */
    @GetMapping("/scanCode/getPool/{uuid}")
    public R<?> getPool(@PathVariable String uuid) {
        R<?> pool = tokenService.getPool(uuid);
        return pool;
    }

    /**
     * 扫码成功进入确认登录页面记录状态
     *
     * @param uuid
     * @return
     */
    @GetMapping(value = "/scanCode/confirm/{uuid}")
    public R<?> scanCode(@PathVariable String uuid) {
        return tokenService.scanCode(uuid);
    }


    /**
     * app扫码登录
     *
     * @param form
     * @return
     */
    @PostMapping("/scanCode/phoneScanLogin")
    public R<?> phoneScanLogin(@RequestBody LoginBody form) {
        return loginService.phoneScanLogin(form);
    }
}
